@@ -13,7 +13,7 @@ use Masterkey::Admin::Config;
 use Masterkey::Admin::Torus;
 
 our @ISA = qw();
-our $VERSION = '3.24';
+our $VERSION = '3.25';
 
 sub new {
     my $class = shift();
@@ -253,6 +253,11 @@ sub fieldname {
 
     return $val if defined $val;
 
+    # Otherwise, return a non-mangled version for completely unknown
+    # fields, as these will often have been added by hand: see
+    # MKA-108.
+    return $name if !$this->conf("field.$name._exists");
+
     $name =~ s/([A-Z])/ $1/g;
     return ucfirst($name);
 }
@@ -263,27 +268,40 @@ sub fieldname {
 #
 sub make_record {
     my $this = shift();
-    my($identity, $user, %data) = @_;
+    my($identity, $user, $master, %data) = @_;
 
     my $torus = $this->torus();
     my @fields = $this->configObject()->profile()->fields();
-    foreach my $ref (@fields) {
-	my($name, $mandatory) = @$ref;
-	my $val = $data{$name};
-	if ($mandatory && (!defined $val || $val eq "")) {
-	    return undef;
+
+    if (!$master) {
+	foreach my $ref (@fields) {
+	    my($name, $mandatory) = @$ref;
+	    my $val = $data{$name};
+	    if ($mandatory && (!defined $val || $val eq "")) {
+		return undef;
+	    }
 	}
     }
 
-    my $id = $torus->add($identity, undef, $user, %data);
+    my $id = $torus->add($master, $identity, undef, $user, %data);
+
+    my($caption, $rendered);
+    if ($master) {
+	$caption = 'master record';
+	$rendered = '{' . join(', ', map { $_ . '="' . $data{$_} . '"' } sort keys %data) . '}';
+    } else {
+	$caption = 'record';
+	$rendered = '"' . $data{displayName} . '"';
+    }
+    warn "added $caption $rendered";
+
     if ($this->conf("email.event.newRecord") ||
 	$this->conf("sendEmail.new")) {
 	$this->send_email(($this->conf("email.toAdmin") ||
 			   $this->conf("adminEmail")),
-			  "New record added",
+			  ("New $caption added"),
 			  "User '" . $user->displayName() . "' " .
-			  "added record '" . $data{displayName} .
-			  "' (id=$id)");
+			  "added record " . $rendered . " (id=$id)");
     }
 
     $torus->make_realms_for_record(%data);
@@ -338,14 +356,14 @@ sub dump {
 
 
 # The Torus often returns rather verbose error messages which we don't
-# need to show to the user in detail. This function trim them of the
-# extraneous detail.
+# need to show to the user in detail. This function trims them of the
+# extraneous detail, retaining information only if $debug is set.
 #
 sub exception2error {
     my $this = shift();
-    my($s) = @_;
+    my($s, $debug) = @_;
 
-    $s =~ s/Stack:.*//s;
+    $s =~ s/Stack:.*//s if !$debug;
     $s =~ s/.*Parsing query failed -(.*) at character \d+\)/Bad query:$1/s;
     return qq[   <p class="error">$s</p>\n];
 }
