@@ -209,7 +209,6 @@ sub _listURI {
 
     my %params = ();
     $params{layers} = "override,original,final";
-    $params{allowTimedRefresh} = "true";
     $params{query} = $query if defined $query;
     $params{facets} = $facets if defined $facets;
     $params{start} = $start if defined $start;
@@ -407,22 +406,32 @@ sub update {
     $this->_put($uri, $xml);
 }
 
+#### update to allow creating master records
 sub add {
     my $this = shift();
-    my($realm, $worldId, $user, %data) = @_;
+    my($master, $realm, $worldId, $user, %data) = @_;
 
     die "add invoked on 'world'" if defined $realm && $realm eq "world";
-    $this->_logTorusOp("add", realm=>$realm, worldId=>$worldId, user=>$user);
+    $this->_logTorusOp("add", master=>$master, realm=>$realm, worldId=>$worldId, user=>$user);
+
+    if (!$master) {
+	$this->_check_unique($realm, $user, %data);
+    }
 
-    $this->_check_unique($realm, $user, %data);
     my %extra = ();
     if ($this->admin()->conf("constrainIdentity")) {
 	%extra = (identityId => $user->field("identityId"));
     }
 
     $data{worldId} = $worldId if defined $worldId;
-    my $xml = $this->_makeXML(undef, {}, %data, %extra);
-    my $uri = $this->recordsURI($realm, undef, $user, 1);
+    my($uri, $xml);
+    if ($master) {
+	$uri = $this->mastersURI($realm);
+	$xml = _master_xml(undef, "XXX", "cql.allRecords=1", {}, %data);
+    } else {
+	$uri = $this->recordsURI($realm, undef, $user, 1);
+	$xml = $this->_makeXML(undef, {}, %data, %extra);
+    }
     my $res = $this->_post($uri, $xml);
 
     # Content is now empty, but the Location header has something like:
@@ -682,7 +691,7 @@ sub make_realms_for_record {
 	    or die "realmOnNew.$i.type not specified";
 
 	my $uri = $this->_substitute($realm, \%data);
-	$this->create_realm($uri, $type);
+	$this->create_realm($uri, $type, 0);
 
 	my $j = 1;
 	while (1) {
@@ -725,9 +734,12 @@ sub remove_realms_for_record {
 
 sub create_realm {
     my $this = shift();
-    my ($uri, $type) = @_;
+    my ($uri, $type, $autoInherit) = @_;
+
+    my $xml = qq[<realm type="$type" name="this will be ignored"];
+    $xml .= ' autoInherit="yes"' if $autoInherit;
+    $xml .= '>';
 
-    my $xml = qq[<realm type="$type" name="this will be ignored">];
     if ($type eq "searchable") {
 	$xml .= qq[
   <matchKey>
@@ -899,6 +911,7 @@ sub parentsURI {
 
 ### The "master" code here is disturbingly similar to the "parent" code.
 
+### This is never used
 sub updateMaster {
     my $this = shift();
     my($realm, $id, $label, $matchQuery, $goneHash, %data) = @_;
